#include "myevent.h"
#include <cairomm/context.h>
#include <gtkmm/label.h>
#include <iostream>

constexpr int area_side(200);

MyEvent::MyEvent():
	m_Main_Box(Gtk::Orientation::HORIZONTAL, 0),
	m_Panel_Box(Gtk::Orientation::VERTICAL, 2),
	m_Buttons_Box(Gtk::Orientation::VERTICAL, 2),
	m_Panel_Frame("General"),
	m_Button_Draw_Clear("Clear"),
	draw(true), thick_linewidth(true)
{
	// init layout
	set_title("Keyboard events");
	set_child(m_Main_Box);

	m_Main_Box.append(m_Panel_Box);
	m_Main_Box.append(m_Area);
	
	m_Panel_Box.append(m_Panel_Frame);
	m_Panel_Frame.set_child(m_Buttons_Box);
	m_Buttons_Box.append(m_Button_Draw_Clear);
	
	// init buttons signal handlers	
    m_Button_Draw_Clear.signal_clicked().connect(
		     sigc::mem_fun(*this, &MyEvent::on_button_clicked_draw_clear));
		
	// init drawing sub-window parameters
	m_Area.set_content_width(area_side);
	m_Area.set_content_height(area_side);
	m_Area.set_expand();
	m_Area.set_draw_func(sigc::mem_fun(*this, &MyEvent::on_draw));

    // Handling Keyboard Events.
    auto controller = Gtk::EventControllerKey::create();
    controller->signal_key_pressed().connect(
                sigc::mem_fun(*this, &MyEvent::on_window_key_pressed), false);
    add_controller(controller);
}

void MyEvent::on_button_clicked_draw_clear()
{
	std::cout << "changing drawing state through the button" << std::endl;
	draw = !draw;
	if(draw) 	m_Button_Draw_Clear.set_label("CLEAR!");
	else 		m_Button_Draw_Clear.set_label("DRAW !");
	m_Area.queue_draw();
}

void MyEvent::on_draw(const Cairo::RefPtr<Cairo::Context>& cr, int width, int height)
{
	if(draw)
	{
		// coordinates for the center of the window
		int xc, yc;
		xc = width / 2;
		yc = height / 2;

		if(thick_linewidth) 		cr->set_line_width(10.0);
		else						cr->set_line_width(2.0);
		
		// draw red lines out from the center of the window
		cr->set_source_rgb(0.8, 0.0, 0.0);
		cr->move_to(0, 0);
		cr->line_to(xc, yc);
		cr->line_to(0, height);
		cr->move_to(xc, yc);
		cr->line_to(width, yc);
		cr->stroke();
	}
	else
	{
		std::cout << "Nothing to draw !" << std::endl;
	}
}

bool MyEvent::on_window_key_pressed(guint keyval, guint, Gdk::ModifierType state)
{
	switch(gdk_keyval_to_unicode(keyval))
	{
	  case 'c':
	  case 'd':
		std::cout << "changing drawing state through the keyboard" << std::endl;
		draw = !draw;
		if(draw) 	m_Button_Draw_Clear.set_label("CLEAR!");
		else 		m_Button_Draw_Clear.set_label("DRAW !");
		m_Area.queue_draw();
		return true;
	  case 'w':
		std::cout << " changing state variable thick_linewidth" << std::endl;
		thick_linewidth= !thick_linewidth;
		m_Area.queue_draw();
		return true;
	  case 'q':
		std::cout << "Quit" << std::endl;
		hide(); //ou  exit(0);
		return true;
	}
    //the event has not been handled
    return false;
}
