#include "myevent.h"
#include <cairomm/context.h>
#include <gtkmm/label.h>
#include <iostream>
using namespace std;

constexpr int area_side(200);

MyEvent::MyEvent():
	m_Main_Box(Gtk::Orientation::HORIZONTAL, 0),
	m_Panel_Box(Gtk::Orientation::VERTICAL, 2),
	m_Buttons_Box(Gtk::Orientation::VERTICAL, 2),
	m_Panel_Frame("General"),
	m_Button_Move_xc_yc("Move xc_yc"),
	move_xc_yc(false)
{
	// init layout
	set_title("Mouse events");
	set_child(m_Main_Box);

	m_Main_Box.append(m_Panel_Box);
	m_Main_Box.append(m_Area);
	
	m_Panel_Box.append(m_Panel_Frame);
	m_Panel_Frame.set_child(m_Buttons_Box);
	m_Buttons_Box.append(m_Button_Move_xc_yc);
	
	// init buttons signal handlers	
    m_Button_Move_xc_yc.signal_clicked().connect(
		     sigc::mem_fun(*this, &MyEvent::on_button_clicked_move_xc_yc));
		
	// init drawing sub-window parameters
	m_Area.set_content_width(area_side);
	m_Area.set_content_height(area_side);
	m_Area.set_expand();
	m_Area.set_draw_func(sigc::mem_fun(*this, &MyEvent::on_draw));
	xc = yc = area_side/2;
	
	set_mouse_controller();
}

void MyEvent::on_button_clicked_move_xc_yc()
{
	cout << "changing use of mouse move data" << endl;
	move_xc_yc = !move_xc_yc;
	if(move_xc_yc ) m_Button_Move_xc_yc.set_label("Do Not Move!");
	else 			m_Button_Move_xc_yc.set_label("Move (xc,yc)");
	m_Area.queue_draw();
}

void MyEvent::on_draw(const Cairo::RefPtr<Cairo::Context>& cr, 
                      int width, int height)
{
	cr->set_line_width(10.0);
	
	// draw red lines linked to (xc, yc) 
	cr->set_source_rgb(0.8, 0.0, 0.0);
	cr->move_to(0, 0);
	cr->line_to(xc, yc);
	cr->line_to(0, height);
	cr->move_to(xc, yc);
	cr->line_to(width, yc);
	cr->stroke();
}

void MyEvent::set_mouse_controller()
{
    auto left_click = Gtk::GestureClick::create();
    auto right_click = Gtk::GestureClick::create();
    auto move = Gtk::EventControllerMotion::create();

    left_click->set_button(GDK_BUTTON_PRIMARY);
    right_click->set_button(GDK_BUTTON_SECONDARY);

    left_click->signal_pressed().connect(
        sigc::mem_fun(*this, &MyEvent::on_drawing_left_click));
    right_click->signal_pressed().connect(
        sigc::mem_fun(*this, &MyEvent::on_drawing_right_click));
    move->signal_motion().connect(
        sigc::mem_fun(*this, &MyEvent::on_drawing_move));

    m_Area.add_controller(left_click);
    m_Area.add_controller(right_click);
    m_Area.add_controller(move);
}

void MyEvent::on_drawing_left_click(int n_press, double x, double y)
{
	cout << "mouse left clic detected and processed" << endl;
	cout << "x = " << x << "   y = " << y << endl;
	xc = x;
	yc = y;
	m_Area.queue_draw();
}

void MyEvent::on_drawing_right_click(int n_press, double x, double y)
{
	cout << "mouse right clic detected and processed" << endl;
	move_xc_yc = !move_xc_yc;
	if(move_xc_yc ) m_Button_Move_xc_yc.set_label("Do Not Move!");
	else 			m_Button_Move_xc_yc.set_label("Move (xc,yc)");
	m_Area.queue_draw();
}

void MyEvent::on_drawing_move(double x, double y)
{
	static char c('A');
	cout << c << " => mouse move detected and processed" << endl;
	if(++c >'Z') c= 'A'; // reset 
	
	if(move_xc_yc)
	{
		if(x > 0 and x < m_Area.get_width())
			xc = x;
			
		if(y > 0 and x < m_Area.get_height())
			yc = y;
	}
	else
	{
		xc = m_Area.get_width()/2 ;
		yc = m_Area.get_height()/2 ;
	}
	m_Area.queue_draw();
}

